-- Copyright (C) by Jiancong Ni

local math = require "math"
local table = table
local string = string
local type = type
local _M = { _VERSION = '0.01' }
local mt = { __index = _M }

module(...)

local fix_char =
{'0','1','2','3','4','5','6','7','8','9',
'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
}

local fix_char_value =
{[48]=0,[49]=1,[50]=2,[51]=3,[52]=4,[53]=5,[54]=6,[55]=7,[56]=8,[57]=9,
[65]=10,[66]=11,[67]=12,[68]=13,[69]=14,[70]=15,[71]=16,[72]=17,[73]=18,
[74]=19,[75]=20,[76]=21,[77]=22,[78]=23,[79]=24,[80]=25,[81]=26,[82]=27,
[83]=28,[84]=29,[85]=30,[86]=31,[87]=32,[88]=33,[89]=34,[90]=35,}

local function to36(number)
	if not number or type(number) ~= "number" then return nil end
	local stack = {}
	local result = ""
	local rest = number
	while rest ~= 0 do
		local x = rest - math.floor(rest / 36) * 36
		table.insert(stack, fix_char[x + 1])
		rest = math.floor(rest / 36);
	end
	for i = #stack, 1, -1 do
		result = result .. stack[i]
	end
	if #result == 0 then result = "0" end
	return result
end
	
local function to10(str)
	if not str or type(str) ~= "string" then return nil end
	local multiple = 1
	local result = 0
	local c
	str = string.upper(str)
	for i = #str, 1, -1 do
		c = fix_char_value[string.byte(str, i)]
		if not c then return nil end
		result = result + c * multiple
		multiple = multiple * 36
	end
	return result
end

_M.to36 = to36
_M.to10 = to10

return _M
